package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PurchaseCatalogueDTO;
import digital.laboratory.platform.reagent.entity.CatalogueDetails;
import digital.laboratory.platform.reagent.entity.PurchaseCatalogue;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.mapper.PurchaseCatalogueMapper;
import digital.laboratory.platform.reagent.service.CatalogueDetailsService;
import digital.laboratory.platform.reagent.service.PurchaseCatalogueService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.PurchaseCatalogueVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;

/**
 * (采购目录)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购目录) 服务实现类
 */
@SuppressWarnings("all")
@Service


public class PurchaseCatalogueServiceImpl extends ServiceImpl<PurchaseCatalogueMapper, PurchaseCatalogue> implements PurchaseCatalogueService {
    @Autowired
    private CatalogueDetailsService catalogueDetailsService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired
    private CategoryTableServiceImpl typeTableService;

    @Override//新增采购目录
    @Transactional(rollbackFor = Exception.class)
    public PurchaseCatalogue addCatalogue(DLPUser dlpUser, List<PurchaseCatalogueDTO> purchaseCatalogueDTOList) {
        PurchaseCatalogue purchaseCatalogue = new PurchaseCatalogue();
        purchaseCatalogue.setPurchaseCatalogueId(IdWorker.get32UUID().toUpperCase());
        purchaseCatalogue.setStatus(0);
        List<CatalogueDetails> catalogueDetailsList = new ArrayList<>();
        Integer i = 1;
        for (PurchaseCatalogueDTO purchaseCatalogueDto : purchaseCatalogueDTOList) {
            CatalogueDetails catalogueDetails = new CatalogueDetails();
            BeanUtils.copyProperties(purchaseCatalogueDto, catalogueDetails);
            catalogueDetails.setCatalogueDetailsId(IdWorker.get32UUID().toUpperCase());
            catalogueDetails.setPurchaseCatalogueId(purchaseCatalogue.getPurchaseCatalogueId());
            catalogueDetails.setPurchaseCatalogueNumber(LocalDateTimeUtil.format(LocalDateTime.now(), "yyyy") + "-" + i.toString());
            i++;
            catalogueDetailsList.add(catalogueDetails);
            typeTableService.addSpecies(catalogueDetails.getCategory(), catalogueDetails.getSpecies());
        }

        if (this.save(purchaseCatalogue) & catalogueDetailsService.saveBatch(catalogueDetailsList)) {
            return purchaseCatalogue;
        } else return null;
    }


    @Override//修改采购目录
    public CatalogueDetails editCatalogue(PurchaseCatalogueDTO purchaseCatalogueDTO) {
        CatalogueDetails catalogueDetails = catalogueDetailsService.getById(purchaseCatalogueDTO.getCatalogueDetailsId());
        BeanUtils.copyProperties(purchaseCatalogueDTO, catalogueDetails);
        if (catalogueDetailsService.updateById(catalogueDetails)) {
            return catalogueDetails;
        } else return null;
    }

    @Override//删除采购目录
    @Transactional(rollbackFor = Exception.class)
    public Boolean delCatalogue(String purchaseCatalogueId) {
        List<CatalogueDetails> list = catalogueDetailsService.list(Wrappers.<CatalogueDetails>lambdaQuery().eq(CatalogueDetails::getPurchaseCatalogueId, purchaseCatalogueId));
        if (list != null && list.size() > 0) {
            return catalogueDetailsService.removeBatchByIds(list) && this.removeById(purchaseCatalogueId);
        } else {
            return this.removeById(purchaseCatalogueId);
        }
    }

    @Override
    public CatalogueDetails addDetails(PurchaseCatalogueDTO purchaseCatalogueDTO) {
        CatalogueDetails catalogueDetails = new CatalogueDetails();
        BeanUtils.copyProperties(purchaseCatalogueDTO, catalogueDetails);
        String yyyy = LocalDateTimeUtil.format(LocalDateTime.now(), "yyyy");
        catalogueDetails.setPurchaseCatalogueNumber(yyyy + "-" + catalogueDetailsService.list(Wrappers.<CatalogueDetails>lambdaQuery()
                .eq(CatalogueDetails::getPurchaseCatalogueId, purchaseCatalogueDTO.getPurchaseCatalogueId())).size() + 1);
        if (catalogueDetailsService.save(catalogueDetails)) {
            return catalogueDetails;
        } else return null;
    }

    @Override
    public IPage<PurchaseCatalogueVO> getPurchaseCatalogueVOPage(IPage<PurchaseCatalogue> page, QueryWrapper<PurchaseCatalogue> qw) {
        IPage<PurchaseCatalogueVO> purchaseCatalogueVOPage = baseMapper.getPurchaseCatalogueVOPage(page, qw);
        List<PurchaseCatalogueVO> records = purchaseCatalogueVOPage.getRecords();
        for (PurchaseCatalogueVO record : records) {
            List<CatalogueDetails> list = catalogueDetailsService.list(Wrappers.<CatalogueDetails>lambdaQuery()
                    .eq(CatalogueDetails::getPurchaseCatalogueId, record.getPurchaseCatalogueId()));
            record.setQuantity(list.size());
        }
        return purchaseCatalogueVOPage;
    }

    @Override
    public List<PurchaseCatalogueVO> getPurchaseCatalogueVOList(QueryWrapper<PurchaseCatalogue> qw) {
        return baseMapper.getPurchaseCatalogueVOList(qw);
    }

    @Override//通过ID查询
    public PurchaseCatalogueVO getPurchaseCatalogueVO(String purchaseCatalogueId) {
        PurchaseCatalogueVO purchaseCatalogueVO = baseMapper.getPurchaseCatalogueVO(purchaseCatalogueId);
        List<CatalogueDetails> catalogueDetailsList = catalogueDetailsService.getCatalogueDetailsList(purchaseCatalogueId);
        purchaseCatalogueVO.setCatalogueDetailsListList(catalogueDetailsList);
        purchaseCatalogueVO.setQuantity(catalogueDetailsList.size());
        return purchaseCatalogueVO;
    }

    @Override//提交目录
    @Transactional
    public PurchaseCatalogue commitById(List<PurchaseCatalogueDTO> purchaseCatalogueDTOList, DLPUser dlpUser) {
        Set<String> permissions = dlpUser.getPermissions();
        if (!StrUtil.isNotBlank(purchaseCatalogueDTOList.get(0).getPurchaseCatalogueId())) {
            PurchaseCatalogue purchaseCatalogue = this.addCatalogue(dlpUser, purchaseCatalogueDTOList);
            if (permissions.contains("reagent_purchase_catalogue_primary")) {
                purchaseCatalogue.setStatus(2);
            } else {
                purchaseCatalogue.setStatus(1);
            }
            purchaseCatalogue.setCommitTime(LocalDateTime.now());
            if (this.updateById(purchaseCatalogue)) {
                return purchaseCatalogue;
            } else return null;

        } else {
            PurchaseCatalogue purchaseCatalogue = this.getById(purchaseCatalogueDTOList.get(0).getPurchaseCatalogueId());
            if (permissions.contains("reagent_purchase_catalogue_primary")) {
                purchaseCatalogue.setStatus(2);
            } else {
                purchaseCatalogue.setStatus(1);
            }
            purchaseCatalogue.setCommitTime(LocalDateTime.now());
            if (this.updateById(purchaseCatalogue)) {
                return purchaseCatalogue;
            } else return null;
        }
    }

    @Override//审核
    @Transactional
    public PurchaseCatalogue primaryAuditCatalogue(AuditAndApproveDTO auditAndApproveDto, DLPUser dlpUser) {
        PurchaseCatalogue purchaseCatalogue = this.getById(auditAndApproveDto.getUuId());
        purchaseCatalogue.setAuditOpinion(auditAndApproveDto.getAuditOpinion());
        purchaseCatalogue.setAuditResult(auditAndApproveDto.getAuditResult());
        purchaseCatalogue.setAuditorId(dlpUser.getId());
        purchaseCatalogue.setAuditTime(LocalDateTime.now());
        if (purchaseCatalogue.getAuditResult() == true) {
            //审核通过后，将采购目录添加的试剂耗材，录入至试剂耗材类
            List<CatalogueDetails> list = catalogueDetailsService.list(Wrappers.<CatalogueDetails>lambdaQuery()
                    .eq(CatalogueDetails::getPurchaseCatalogueId, purchaseCatalogue.getPurchaseCatalogueId()));

            //目录中的试剂耗材是否存在试剂耗材类中，有则不管，无则添加
            for (CatalogueDetails catalogueDetails : list) {
                ReagentConsumables reagentConsumable = reagentConsumablesService.getOne(Wrappers.<ReagentConsumables>query()
                        .eq("reagent_consumable_name", catalogueDetails.getReagentConsumableName())
                        .eq("brand", catalogueDetails.getBrand())
                        .eq("category", catalogueDetails.getCategory())
                        .eq("specification_and_model", catalogueDetails.getSpecificationAndModel())
                        .eq("unit_price", catalogueDetails.getUnitPrice())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getStandardValueOrPurity()), "standard_value_or_purity", catalogueDetails.getStandardValueOrPurity())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getCasNumber()), "cas_number", catalogueDetails.getCasNumber())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getSpecies()), "species", catalogueDetails.getSpecies())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getEnglishName()), "english_name", catalogueDetails.getEnglishName())
                        .eq(catalogueDetails.getPackagedCopies() != null, "packaged_copies", catalogueDetails.getPackagedCopies())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getAlias()), "alias", catalogueDetails.getAlias())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getPurityGrade()), "purity_grade", catalogueDetails.getPurityGrade())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getDeviationOrUncertainty()), "deviation_or_uncertainty", catalogueDetails.getDeviationOrUncertainty())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getMinimumUnit()), "minimum_unit", catalogueDetails.getMinimumUnit())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getStorageCondition()), "storage_condition", catalogueDetails.getStorageCondition())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getRemark()), "remark", catalogueDetails.getRemark())
                        .eq(StrUtil.isNotBlank(catalogueDetails.getCode()), "code", catalogueDetails.getCode())
                );

                if (reagentConsumable == null) {
                    ReagentConsumables addReagentConsumables = reagentConsumablesService.addReagentConsumables(catalogueDetails);
                    catalogueDetails.setReagentConsumableId(addReagentConsumables.getReagentConsumableId());
                    catalogueDetailsService.updateById(catalogueDetails);
                } else {
                    catalogueDetails.setReagentConsumableId(reagentConsumable.getReagentConsumableId());
                    catalogueDetailsService.updateById(catalogueDetails);
                }
            }
            purchaseCatalogue.setStatus(6);
        } else {
            purchaseCatalogue.setStatus(-3);
        }
        if (this.updateById(purchaseCatalogue)) {
            return purchaseCatalogue;
        } else return null;
    }

    @Override
    public PurchaseCatalogue releaseCatalogue(String purchaseCatalogueId) {
        PurchaseCatalogue purchaseCatalogue = this.getById(purchaseCatalogueId);
        purchaseCatalogue.setStatus(4);
        if (this.updateById(purchaseCatalogue)) {
            return purchaseCatalogue;
        } else return null;
    }

    @Override//查询已发布的采购目录
    public List<CatalogueDetails> getList(String name, Integer category) {
        List<PurchaseCatalogue> list = this.list(Wrappers.<PurchaseCatalogue>lambdaQuery().eq(PurchaseCatalogue::getStatus, 4)
                .orderByDesc(PurchaseCatalogue::getUpdateTime));
        if (list == null & list.size() == 0) {
            throw new RuntimeException(String.format("未存在已发布的采购目录"));
        }
        PurchaseCatalogue purchaseCatalogue = list.get(0);
        List<CatalogueDetails> catalogueDetailsList = catalogueDetailsService.list(Wrappers.<CatalogueDetails>lambdaQuery()
                .eq(CatalogueDetails::getPurchaseCatalogueId, purchaseCatalogue.getPurchaseCatalogueId())
                .like(StrUtil.isNotBlank(name), CatalogueDetails::getReagentConsumableName, name)
                .eq(category != null && category == 1, CatalogueDetails::getCategory, "试剂")
                .eq(category != null && category == 2, CatalogueDetails::getCategory, "耗材")
                .eq(category != null && category == 3, CatalogueDetails::getCategory, "标准物质")
        );
        Collections.sort(catalogueDetailsList, new Comparator<CatalogueDetails>() {
            @Override
            public int compare(CatalogueDetails o1, CatalogueDetails o2) {
                return Integer.valueOf(o1.getPurchaseCatalogueNumber().substring(5)) - Integer.valueOf(o2.getPurchaseCatalogueNumber().substring(5));
            }
        });
        return catalogueDetailsList;
    }

    @Override
    //导入方法，验证填写内容是否有误
    public PurchaseCatalogueVO getImport(List<PurchaseCatalogueDTO> purchaseCatalogueDTOList) {
        int i = 1;
        String message = "";
        try {
            for (PurchaseCatalogueDTO purchaseCatalogueDTO : purchaseCatalogueDTOList) {

                if (StrUtil.isNotBlank(purchaseCatalogueDTO.getBrand()) &
                        StrUtil.isNotBlank(purchaseCatalogueDTO.getCategory()) &
                        (purchaseCatalogueDTO.getPackagedCopies()) != null &
                        StrUtil.isNotBlank(purchaseCatalogueDTO.getReagentConsumableName()) &
                        StrUtil.isNotBlank(purchaseCatalogueDTO.getSpecies()) &
                        StrUtil.isNotBlank(purchaseCatalogueDTO.getSpecificationAndModel()) &
                        purchaseCatalogueDTO.getUnitPrice() != 0 &
                        StrUtil.isNotBlank(purchaseCatalogueDTO.getStorageCondition()) &
                        StrUtil.isNotBlank(purchaseCatalogueDTO.getMinimumUnit())
                ) {
                    i++;
                } else {
                    message = message + "\n" + "第" + i + "条数据有误，请改正后重新导入";
                    i++;
                }
            }

        } catch (RuntimeException e) {
            throw new RuntimeException(message);
        }
        if (StrUtil.isNotBlank(message)) {
            throw new RuntimeException(message);
        }

        PurchaseCatalogueVO purchaseCatalogueVO = new PurchaseCatalogueVO();
        List<CatalogueDetails> catalogueDetailsList = new ArrayList<>();
        for (PurchaseCatalogueDTO purchaseCatalogueDto : purchaseCatalogueDTOList) {
            CatalogueDetails catalogueDetails = new CatalogueDetails();
            BeanUtils.copyProperties(purchaseCatalogueDto, catalogueDetails);
            catalogueDetailsList.add(catalogueDetails);
        }
        purchaseCatalogueVO.setCatalogueDetailsListList(catalogueDetailsList);
        return purchaseCatalogueVO;
    }

}
